/**
* Klasse, die Bildschirmpräsentation steuert
*/
function Slideshow(
    objStepByStep,  // Zuständige Instanz von StepByStep
    lessonData      // Objekt mit Informationen über Präsentation
) {
    this.objStepByStep = objStepByStep;
    this.data = lessonData;
    this._loadSvgs();
}

// Zuständige Instanz von StepByStep
Slideshow.prototype.objStepByStep = null;

// Informationen zur Präsentation
Slideshow.prototype.data = {};

// Wie viele Dateien müssen noch geladen werden?
Slideshow.prototype.countStillLoading = 0;

/**
* Die verwendeten SVG-Dateien laden
* @return undefined
*/
Slideshow.prototype._loadSvgs = function() {
    for (var n = 0; n < this.data.slideinfos.length; ++n) {
        if (this.data.slideinfos[n].src) {
            (function(src, self, index) {
                var oReq = new XMLHttpRequest();
                oReq.overrideMimeType('text/xml');
                oReq.addEventListener("load", function() {
                    self.importSvg(oReq.responseXML, index);
                });
                oReq.addEventListener("error", function() {
                    self.objStepByStep.showMsg('„' + src + '“ konnte wegen eines unerwarteten Fehlers leider nicht geladen werden…');
                });
                oReq.addEventListener("abort", function() {
                    self.objStepByStep.showMsg('„' + src + '“ kann nicht geladen werden, weil du den Ladevorgang abgebrochen hast.');
                });
                oReq.open('GET', ((self.data.basedir && !self.data.slideinfos[index].external) ? (self.data.basedir + '/') : '') + src);
                oReq.send();
            })(this.data.slideinfos[n].src, this, n);
            ++this.countStillLoading;
        }
    }
    if (!this.countStillLoading)
        this.objStepByStep.showMsg('Präsentation enthält keinen anzeigbaren Inhalt…');
}

/**
* Eine geladene Folie oder Overlay importieren
* @return undefined
*/
Slideshow.prototype.importSvg = function(
    xml,        // Document mit SVG
    index       // Index des zugehörigen Schrittes
) {
    var svg = xml.documentElement;
    var node = createSvg('g');
    var transform = false;
    if (svg.hasAttribute('viewBox')) {
        var parts = svg.getAttribute('viewBox').split(' ');
        if (parts.length == 4) {
            var a = [];
            if (parts[0])
                a.push('translateX(-' + parts[0] * ((parts[2] - parts[0]) / this.data.width) + 'px)');
            if (parts[1])
                a.push('translateY(-' + parts[1] * ((parts[3] - parts[1]) / this.data.height) + 'px)');
            if (parts[2])
                a.push('scaleX(' + (this.data.width / (parts[2] - parts[0])) + ')');
            if (parts[3])
                a.push('scaleY(' + (this.data.height / (parts[3] - parts[1])) + ')');
            transform = a.join(' ');
            node.style.transform = transform;
        }
    }
    for (var child = svg.firstChild; child; child = child.nextSibling)
        node.appendChild(node.ownerDocument.importNode(child, true));
    this.data.slideinfos[index].node = node;
    // Referenzen korrigieren
    if (this.data.basedir) {
        var nodesImages = node.getElementsByTagName('image');
        var href;
        for (var n = 0; n < nodesImages.length; ++n) {
            href = nodesImages[n].getAttributeNS('http://www.w3.org/1999/xlink', 'href');
            if (href)
                nodesImages[n].setAttributeNS('http://www.w3.org/1999/xlink', 'href', this.data.basedir + '/' + href);
        }
    }
    // Nutzen weitere Schritte Ebenen von node?
    if (this.data.slideinfos[index].id) {
        var child, countLayer, ripped;
        for (var n = 0; n < this.data.slideinfos.length; ++n) {
            if (this.data.slideinfos[index].id != this.data.slideinfos[n].useLayerOf)
                continue;
            countLayer = 0;
            for (child = node.firstChild; child; child = child.nextSibling) {
                if ((child.nodeType != Node.ELEMENT_NODE) || (child.localName.toUpperCase() != 'G') || (++countLayer < 2))
                    continue;
                if (transform) {
                    ripped = createSvg('g');
                    ripped.style.transform = transform;
                    ripped.appendChild(child);
                    child = ripped;
                } else {
                    node.removeChild(child);
                }
                this.data.slideinfos[n].node = createSvg('g');
                this.data.slideinfos[n].node.appendChild(child);
                break;
            }
        }
    }
    --this.countStillLoading;
    if (this.countStillLoading <= 0)
        this.establish();
}

// Präsentationsfläche
Slideshow.prototype.area = null;

// SVGGroupElement mit aktueller Folie
Slideshow.prototype.nodeActive = null;

// SVGGroupElement mit vorheriger Folie
Slideshow.prototype.nodeLast = null;

// Aktuell laufende Animationen
Slideshow.prototype.arrAnimations = [];

/**
* Präsentationsfläche erstellen, anzeigen, Navigationsleiste einblenden und Kommunikation mit LMS starten
* @return undefined
*/
Slideshow.prototype.establish = function() {
    this.arrAnimations = [];
    this.area = createSvg('svg', null, { width: this.data.width + "mm", height: this.data.height + "mm", version: "1.1", viewBox: "0 0 " + this.data.width + " " + this.data.height });
    this.objStepByStep.establishContent(this.area, this.data.slideinfos);
}

/**
* Aktuelle Animationen abbrechen
* @return undefined
*/
Slideshow.prototype.stopAnimations = function() {
    var o;
    while (this.arrAnimations.length) {
        o = this.arrAnimations.pop();
        try {
            o.finish();
        } catch {}
    }
}

/**
* Eine Folie zusammenbauen
* @return SVGGroupElement   Folie
*/
Slideshow.prototype.buildSlide = function(
    index       // Index des zugehörigen Schrittes in this.data.slideinfos
) {
    var node = createSvg('g', null, { 'class': 'slide' });
    var n = index;
    while ((n > 0) && this.data.slideinfos[n] && !this.data.slideinfos[n].slide)
        --n;
    for (n; n <= index; ++n) {
        if (this.data.slideinfos[n] && this.data.slideinfos[n].node)
            node.appendChild(this.data.slideinfos[n].node.cloneNode(true));
    }
    return node;
}

/**
* Geht ein Wechsel vom Schritt activeOld zum Schritt activeNew mit einem Folienwechsel einher?
* @return Boolean   Folienwechsel?
*/
Slideshow.prototype.isNewSlide = function(
    activeOld,      // Index des Inhalts, der bislang aktiv war (-1 möglich!)
    activeNew       // Index des Inhalts, der neuerdings aktiv sein soll
) {
    if (activeOld < 0)
        return true;
    if (activeNew == activeOld)
        return false;
    if (activeNew >= this.data.slideinfos.length)
        return false;
    if (this.data.slideinfos[activeNew].slide)
        return true;
    var richtung = (activeNew > activeOld) ? +1 : -1;
    for (var n = activeOld + richtung; Math.abs(n - activeNew); n += richtung) {
        if (this.data.slideinfos[n].slide)
            return true;
    }
    return false;
}

/**
* Index der zugehörigen Folie bestimmen
* @return Number        Index der Folie
*/
Slideshow.prototype.getSlideIndex = function(
    index   // Index von Folie oder Overlay
) {
    if (index >= this.data.slideinfos.length)
        index = this.data.slideinfos.length - 1;
    for (index; index > 0; --index) {
        if (this.data.slideinfos[index].slide)
            return index;
    }
    return 0;
}

/**
* Prüft, ob zwei Overlays/Folien zur selben Folie gehören
* @return Boolean   Haben sie eine gemeinsame Folie?
*/
Slideshow.prototype.isSameLayer = function(
    index1,     // Index einer Folie oder eines Overlays
    index2      // Index einer Folie oder eines Overlays
) {
    return (this.getSlideIndex(index1) == this.getSlideIndex(index2));
}

/**
* So scrollen, dass die aktuelle Ebene im sichtbaren Bereich liegt
* @return undefined
*/
Slideshow.prototype.scrollToActive = function(
    content,    // Aktive Gruppe
    navHeight   // Höhe der Navigationsleiste
) {
}

/**
* Overlays von aktueller Folie wieder entfernen (wenn nötig)
* @return Number    Index des jetzt aktiven Schritts
*/
Slideshow.prototype.downgrade = function(
    activeOld,      // Index des Inhalts, der bislang aktiv war (-1 möglich!)
    activeNew       // Index des Inhalts, der neuerdings aktiv sein soll
) {
    if (!this.nodeActive)
        return activeOld;
    for (activeOld; activeOld >= activeNew; --activeOld) {
        if ((this.data.slideinfos[activeOld] && this.data.slideinfos[activeOld].node && this.nodeActive.firstChild) || (activeOld == this.data.slideinfos.length))
            this.nodeActive.removeChild(this.nodeActive.lastChild);
    }
    return activeOld;
}

/**
* Overlays zu aktueller Folie hinzufügen (wenn nötig)
* @return undefined
*/
Slideshow.prototype.upgrade = function(
    activeOld,      // Index des Inhalts, der bislang aktiv war (-1 möglich!)
    activeNew       // Index des Inhalts, der neuerdings aktiv sein soll
) {
    if (!this.nodeActive)
        return activeOld;
    for (++activeOld; activeOld <= activeNew; ++activeOld) {
        if (activeOld == this.data.slideinfos.length)
            createSvg('g', this.nodeActive);
        else if (this.data.slideinfos[activeOld] && this.data.slideinfos[activeOld].node)
            this.nodeActive.appendChild(this.data.slideinfos[activeOld].node.cloneNode(true));
    }
    return activeOld;
}

/**
* Übergang nach Vorgabe animieren
* @return undefined
*/
Slideshow.prototype.addTransition = function(
    nodeNew,        // Neues Element
    nodeOld,        // Optional: Altes Element
    name,           // String mit Animationsnamen
    duration,       // Dauer des Übergangs in Sekunden
    reverse         // Optional: Wird von höherer zu niedrigerer Folie gewechselt?
) {
    var transition = SlideTransition.getByName(name);
    if (!transition)
        return;
    if (reverse)
        transition = transition.reverse();
    try {
        var keyframes = transition.getFramesNew(this.data.width, this.data.height);
        if (keyframes)
            this.arrAnimations.push(nodeNew.animate(keyframes, { duration: 1000 * duration }));
        keyframes = transition.getFramesOld(this.data.width, this.data.height);
        if (keyframes)
            this.arrAnimations.push(nodeOld.animate(keyframes, { duration: 1000 * duration }));
    } catch {}
}

/**
* Erscheinen eines neuen Overlays animieren
* @return undefined
*/
Slideshow.prototype.addOverlayTransition = function(
    nodeOverlay,    // Neues Overlay
    index           // Zugehöriger Index
) {
    var name = this.data.overlayTransition ? this.data.overlayTransition : false;
    var duration = this.data.overlayDuration ? this.data.overlayDuration : 1;
    if (this.data.slideinfos[index]) {
        if (this.data.slideinfos[index].transition)
            name = this.data.slideinfos[index].transition;
        if (this.data.slideinfos[index].duration)
            duration = this.data.slideinfos[index].duration;
        if (name && duration && this.data.slideinfos[index].node)
            this.addTransition(nodeOverlay, null, name, duration, false);
    }
}

/**
* Folienübergang animieren
* @return undefined
*/
Slideshow.prototype.addSlideTransition = function(
    index,          // Zugehöriger Index der neuen Folie
    reverse         // Wird von höherer zu niedrigerer Folie gewechselt?
) {
    index = this.getSlideIndex(index);
    var name = this.data.slideTransition ? this.data.slideTransition : false;
    var duration = this.data.slideDuration ? this.data.slideDuration : 1;
    if (this.data.slideinfos[index]) {
        if (this.data.slideinfos[index].transition)
            name = this.data.slideinfos[index].transition;
        if (this.data.slideinfos[index].duration)
            duration = this.data.slideinfos[index].duration;
    }
    if (name && duration)
        this.addTransition(this.nodeActive, this.nodeLast, name, duration, reverse);
}

/**
* Aktuell aktiven Inhalt korrekt anzeigen
* @return undefined
*/
Slideshow.prototype.changeActive = function(
    activeOld,      // Index des Inhalts, der bislang aktiv war (-1 möglich!)
    activeNew,      // Index des Inhalts, der neuerdings aktiv sein soll
    arrContents     // Sortiertes Array aller Inhalte, die aktiv sein können
) {
    if (!this.nodeActive || (activeOld == -1)) {
        this.nodeActive = this.buildSlide(Math.max(0, Math.min(activeNew, this.data.slideinfos.length - 1)));
        this.area.appendChild(this.nodeActive);
        return;
    }
    if (activeOld == activeNew)
        return;
    this.stopAnimations();
    if (this.isSameLayer(activeOld, activeNew)) {
        // Keine neue Folie
        if (activeOld < this.upgrade(this.downgrade(activeOld, activeNew), activeNew))
            this.addOverlayTransition(this.nodeActive.lastChild, activeNew);
    } else {
        // Neue Folie
        if (this.nodeLast && this.nodeLast.parentNode)
            this.nodeLast.parentNode.removeChild(this.nodeLast);
        this.nodeLast = this.nodeActive;
        this.nodeActive = this.buildSlide(Math.max(0, Math.min(activeNew, this.data.slideinfos.length - 1)));
        this.area.appendChild(this.nodeActive);
        var reverse = (activeNew < activeOld);
        this.addSlideTransition(reverse ? activeOld : activeNew, reverse);
    }
}
